signature PICTURE_LIB = sig
  type shape_def = Shape.shape_def
  type pict_def = Picture.pict_def
  type texture_def = Texture.texture_def
  type point = Geometry.point



  (* (0,0) *)
  val origin : point 
  val everything : shape_def
  val nothing    : shape_def
  val left_of    : (point * point) -> shape_def
  val right_of   : (point * point) -> shape_def
  val transX     : (real * shape_def) -> shape_def
  val transY     : (real * shape_def) -> shape_def
  val transXY    : (real * real * shape_def) -> shape_def
  val scaleX     : (real * shape_def) -> shape_def
  val scaleY     : (real * shape_def) -> shape_def
  val scaleXY    : (real * real * shape_def) -> shape_def


  (* s_1 /\ s_2 /\ s_3 ... /\ everything *)
  val intersect  : shape_def list -> shape_def 
  (* s_1 \/ s_2 \/ s_3 ... \/ nothing *)
  val union      : shape_def list  -> shape_def 
  (* s_1 \ s_2 \ s_3 ... \ nothing *)
  val difference : shape_def list  -> shape_def 

  (* everything \ s *)
  val complement : shape_def -> shape_def

  (* points specified in clockwise order *)    
  val polygon    : point list -> shape_def

  (* isosceles triangle with same height and base centered at *)
  val triangle   : (point * real) -> shape_def

  (* center and width and height *)
  val rect       : (point * real * real) -> shape_def
  (* center and side length *)
  val square     : (point * real) -> shape_def

  (* center and diameter *)
  val circle     : (point * real) -> shape_def

  (* center, diameter *)
  val semi_circle_up   : (point * real) -> shape_def
  val semi_circle_down : (point * real) -> shape_def

  (* shape with known size used of charater layout *)
  type glyph
  val charToGlyph   : char -> glyph
  val hbox          : real -> glyph list -> glyph
  val vbox          : real -> glyph list -> glyph
  val stringToGlyph : string -> glyph
  val glyphShape    : glyph -> shape_def
  val glyphSize     : glyph -> {wd:real,ht:real}

end