structure PictureLib :> PICTURE_LIB =
  struct
    open Shape
    type shape_def = Shape.shape_def
    type pict_def = Picture.pict_def
    type texture_def = Texture.texture_def
    type point = Geometry.point
      

    type glyph  = {wd:real,ht:real,s:shape_def}
    val origin = (0.0,0.0)
    val everything = Everything
    val nothing = Nothing
    fun left_of (p0,p1) = HalfPlane(p0,p1)
    fun right_of (p0,p1) = HalfPlane(p1,p0)

    fun transX (x,s) = Trans((x,0.0),s)
    fun transY (y,s) = Trans((0.0,y),s)
    fun transXY (x,y,s) = Trans((x,y),s)
      
    fun scaleX (a,s) = ScaleXY(a,1.0,s)
    fun scaleY (b,s) = ScaleXY(1.0,b,s)
    fun scaleXY (a,b,s) = ScaleXY(a,b,s)

    fun intersect [] = Everything
      | intersect [x] = x
      | intersect (x::xs) = Intersect(x,intersect xs)

    fun union [] = Nothing
      | union [x] = x
      | union (x::xs) =  Union(x,union xs)

    fun difference [] = Nothing
      | difference [x] = x
      | difference(x::xs) =  Diff(x,difference xs)

    fun complement s = Diff(Everything,s)

    fun polygon (p::ps) = let 
      fun loop(first,[last],acc) = Intersect(acc,HalfPlane(last,first))
	| loop(f,p1::p2::ps,acc) = 
	loop(f,p2::ps,Intersect(acc,HalfPlane(p1,p2)))
	| loop _ = raise Fail "Impossible"
    in loop(p,p::ps,Everything)
    end
      | polygon [] = Nothing
     
    fun triangle ((x,y),s) = let
      val r = s/2.0
    in polygon [(x-r,y-r),(x+r,y-r),(x,y+r)]
    end
  
    fun rect ((x,y),a,b) = let
      val dx = a/2.0
      val dy = b/2.0
    in polygon [(x-dx,y-dy),(x+dx,y-dy),(x+dx,y+dy),(x-dx,y+dy)]
    end

    fun square (p,s) = rect (p,s,s)
    fun circle ((x,y),d) = Elipse((x,y),(d*0.5),(d*0.5))


    fun semi_circle_up ((x,y),d) =
      Intersect(HalfPlane((0.0,y),(1.0,y)),circle((x,y),d))
    fun semi_circle_down ((x,y),d) =
      Intersect(HalfPlane((0.0,y),(~1.0,y)),circle((x,y),d))

    val G = 1.6803
    fun charToGlyph #"H" =
      {wd=1.0,ht=G,s=
       transXY(0.5,0.5*G,
	       difference[rect(origin,1.0,G),
			  union[rect((0.0,G*0.4),0.5,1.0),
				rect((0.0,~G*0.4),0.5,1.0)]])}
      | charToGlyph #"l" =
      {wd=0.25,ht=G,s=
       transXY(0.125,0.5*G,rect(origin,0.25,G))}
      | charToGlyph #"e" =
      {wd=0.5*G,ht=G,s=
        transXY(0.25*G,0.25*G,
		 difference[circle(origin,0.5*G),
		      union[semi_circle_up((0.0,0.05*G),0.2*G),
			    semi_circle_down((0.0,~0.05*G),0.2*G),
			    rect((0.1*G,~0.05*G),0.4*G,0.03*G)]])}
      | charToGlyph #"o" = 
      {wd=0.5*G,ht=G,s=
        transXY(0.25*G,0.25*G,
	 difference[circle(origin,0.5*G),circle(origin,0.25*G)])}
      | charToGlyph #"r" =
      {wd=0.475*G,ht=G,s=
       union[rect((0.125,G*0.25),0.25,G*0.50),
	     difference[shapeOf #"o",left_of((0.0,0.25*G),(~1.0,0.125*G))]]}
      | charToGlyph #"d" =
       {wd=0.5*G,ht=G,s=
	union[shapeOf #"o",transX(0.375*G,shapeOf #"l")]}
      | charToGlyph #"V" = 
       {wd=1.0,ht=G,s=
	transX(0.5,
	       union[polygon[(~0.25,G),(~0.5,G),(~0.125,0.0),(0.125,0.0)],
		     polygon[(~0.125,0.0),(0.125,0.0),(0.5,G),(0.25,G)]])}
      | charToGlyph #"W" = 
       {wd=1.75,ht=G,s=
	union[shapeOf #"V",transX(0.75,shapeOf #"V")]}
      | charToGlyph #" " =  {wd=0.5,ht=G,s=nothing}
      | charToGlyph _ = raise (Fail "glyph not defined")
    and shapeOf c = #s (charToGlyph c)

    fun vbox spc (xs) = let
      fun loop (p,{wd,ht,s}::xs,acc,max_wd) =
	loop (spc+p+ht,xs,Union(Trans((0.0,p),s),acc),Real.max(wd,max_wd))
	| loop (ht,[],acc,max_wd) = {ht=ht,wd=max_wd,s=acc}
    in loop(0.0,(List.rev xs),Nothing,0.0)
    end

    fun hbox spc (xs) = let
      fun loop (p,{wd,ht,s}::xs,acc,max_ht) =
	loop (spc+p+wd,xs,Union(Trans((p,0.0),s),acc),Real.max(ht,max_ht))
	| loop (wd,[],acc,max_ht) = {ht=max_ht,wd=wd,s=acc}
    in loop(0.0,xs,Nothing,0.0)
    end
    fun stringToGlyph s= hbox 0.1 (List.map charToGlyph (String.explode s))
    fun glyphShape {s,wd,ht} = s
    fun glyphSize {wd,ht,s} = {wd=wd,ht=ht}
  end