structure Test =
  struct
    open PictureLib
    val ghw = (vbox 0.1 [stringToGlyph "Hello",stringToGlyph "World"])
    val hw_shape = glyphShape ghw
    val {wd,ht} = glyphSize ghw
    val ll = (~0.5,~0.5)
    val ur = (wd+0.5,ht+0.5)
    val dpi = 75.0
    val hw_area = {dpi=dpi,ll=ll,ur=ur}

    fun pict2image p = let
      val pict = Picture.mkPict p
      fun fg p = Picture.getPixel pict p
    in {bg=Color.white,fg=fg}
    end

    fun pict2ASCII {dpi,ll,ur} p  =  (* fixed dpi *)
      Convert.toASCII {dpi=8.0,ll=ll,ur=ur} (pict2image p)
    fun pict2PNG area p f = Convert.toRGBPNG area (pict2image p) f
    fun pict2PS area p f = Convert.toPS area (pict2image p) f
    fun showIt area p = Convert.display area (pict2image p)

    (** Uncomment if you are having problems with your display. **)
    (* fun showIt area p = pict2PNG area p "ouput.png"
     *)

    (** Uncomment for quick debugging *)
    (* fun showIt area p = pict2ASCII area p *)
    

    val color_orange = Color.fromRGB{r=1.0,g=0.6,b=0.0}
    fun stripes c b k (p1,p2) = let
      val u = Geometry.projectPt(p1,p2)
      val k = Word8.fromInt k
    in (fn p => if (Word8.fromInt(Real.trunc(256.0 * (u p))))< k then c else b)
    end

    val black_texture = Texture.Constant(Color.black)
    val tiger_stripes = 
      stripes color_orange Color.black 128 ((0.0,0.0),(0.05,~0.1))

    val hw_fg =
      Picture.Layer(Texture.Funct tiger_stripes,transXY(0.1,0.1,hw_shape))
    val hw_shadow = 
      Picture.Layer(Texture.Constant(Color.black),hw_shape)
    val hw_pict = Picture.OnTop(hw_fg,hw_shadow)

    fun test_shape s = 
      showIt {dpi=dpi,ll=(~1.5,~1.5),ur=(1.5,1.5)} 
      (Picture.Layer(black_texture,s))

    fun test_display() = test_shape (everything)

    (* tests for 5 (a) *)
    fun test_circle() = test_shape (circle(origin,2.0))
    fun test_half_plane() = test_shape (left_of(origin,(1.0,0.0)))

    (* tests for 5 (b) *)
    fun test_hole() = test_shape (complement(circle(origin,2.0)))
    fun test_semi_circle() = test_shape (semi_circle_up(origin,2.0))

    (* tests for 5 (c) *)
    fun test_letter_e() =  test_shape (glyphShape (charToGlyph #"e"))

    (* tests for 5 (c) and 6 *)
    fun test_hw() = showIt hw_area hw_pict

    (* test for 6 *)
    fun test_opt_hw() = 
      (Shape.optimize := true;
       showIt hw_area hw_pict;
       Shape.optimize := false)


end