structure Convert :> CONVERT =
  struct
    type area = {ll:(real * real),
		 ur:(real * real),
		 dpi:real}
    type shape = (real * real) -> bool
    type image = {bg:Color.color,
		  fg:(real * real) -> Color.color option}
    fun scan {ll=(x_min,y_min),ur=(x_max,y_max),dpi} pf lf = let
      val wd_points = x_max - x_min
      val ht_points = y_max - y_min
      val wd_inches = wd_points / 1.0
      val ht_inches = ht_points / 1.0
      val x_samples = Real.trunc (wd_inches * dpi)
      val y_samples = Real.trunc (ht_inches * dpi)
      val x_delta = wd_points / (wd_inches * dpi)
      val y_delta = ht_points / (ht_inches * dpi)
      fun scan_y(j,y) = let 
	fun scan_x (i,x) = 
	  if i < x_samples then
	    (pf ((i,j),(x,y));scan_x(i+1,x+x_delta))
	  else lf (j)
      in if j > 0 then (scan_x (0,x_min);scan_y(j-1,y - y_delta))
	 else ()
      end
      fun doit () = scan_y(y_samples,y_max)
    in (x_samples,y_samples,doit) 
    end
    fun w2s w = Int.toString (Word8.toInt w)
    fun do_line i = 
      if i mod 10 = 0 then
	TextIO.output(TextIO.stdErr,"line:"^(Int.toString i)^"\n")
      else ()

    fun toPSPoints area out {bg:Color.color,fg} = let
      fun do_color c = let
	val {r,g,b} = Color.toRGB c
	val r = Real.toString r
	val g = Real.toString g
	val b = Real.toString b
	val c = String.concat[r," ",g," ",b," c "]
      in TextIO.output(out,c)
      end
      fun do_draw_point ((x,y),c) = let
	val p = String.concat[Real.toString x," ",Real.toString y," p\n"]
      in do_color c;TextIO.output(out,p)
      end
      fun do_point ((row,col),p) =
	(case fg p of 
	   NONE => ()
	 | SOME c => do_draw_point (p,c))
      val (x_samples,y_samples,doit) = scan area do_point do_line
      val {ll=(x0,y0),ur=(x1,y1),...} = area
      fun c2s x = Real.toString(x*72.0)
      val magic = String.concat
	["%!PS-Adobe-2.0\n",
	 "%%Pages: 1\n",
	 "%%BoundingBox: ",c2s x0," ",
	                   c2s y0," ",
			   c2s x1," ",
	                   c2s y1,"\n",
	 "%%EndComments\n"]
      val header = 
	String.concat[
      magic,
      " /sf 72.0 def\n",
      " sf sf scale\n",
      " /ptsz 1.0 sf  div def\n",
      "/c { setrgbcolor } bind def\n",
      "/p {  ptsz  0 360 arc fill } bind def\n",
       Real.toString x0," ",
       Real.toString y0," translate\n"]
    in TextIO.output(out,header); doit();
       TextIO.output(TextIO.stdErr,"\n")
    end


    fun toASCII area {bg,fg} = let
      fun do_line _ = print "\n"
      fun do_point ((row,col),p) =
	(case fg p of 
	   NONE => print "  "
	 | SOME c => print "##")
      val (x_samples,y_samples,doit) = scan area do_point do_line
      val header = String.concat["\n"]
    in doit(); print "\n"
    end
    
    fun toPPM area out {bg,fg} = let
      fun do_color c = let
	val {r,g,b} = Color.toRGB24 c
      in BinIO.output1(out,r);BinIO.output1(out,g);BinIO.output1(out,b)
      end
      fun do_point ((row,col),p) =
	(case fg p of 
	   NONE => do_color bg
	 | SOME c => do_color c)
      val (x_samples,y_samples,doit) = scan area do_point do_line
      val header = 
	String.concat["P6\n",
        Int.toString x_samples, " ",Int.toString y_samples," 255\n"]
    in BinIO.output(out,Byte.stringToBytes header); doit();
       TextIO.output(TextIO.stdErr,"\n")
    end

    fun toPBM area out shape =
      let fun f p = if shape p then SOME(Color.black) else NONE
      in toPPM area out {bg=Color.white, fg=f}
      end

    fun toGIF doit x p s = let
      val base_name =  OS.Path.base s
      val pnm_file = base_name^".pnm"
      val gif_file = base_name^".gif"
      val outs = BinIO.openOut pnm_file
      val _ = doit x outs p
      val _ = BinIO.closeOut outs
      val cmd = String.concat ["convert ",pnm_file," ",gif_file]
    in OS.Process.system cmd;()
    end

    fun toPNG doit x p s = let
      val base_name =  OS.Path.base s
      val pnm_file = base_name^".pnm"
      val png_file = base_name^".png"
      val outs = BinIO.openOut pnm_file
      val _ = doit x outs p
      val _ = BinIO.closeOut outs
      val cmd = String.concat ["convert ",pnm_file," ",png_file]
    in OS.Process.system cmd;()
    end
    fun toPS doit x p s = let
      val base_name =  OS.Path.base s
      val ps_file = base_name^".ps"
      val outs = TextIO.openOut ps_file
      val _ = doit x outs p
      val _ = TextIO.closeOut outs
    in ()
    end
    fun display area p = let
      val tmp_file =  OS.FileSys.tmpName()
      fun do_it() = let
	val outs = BinIO.openOut tmp_file
	val _ = toPPM area outs p
	val _ = BinIO.closeOut outs
	val cmd = String.concat ["display ",tmp_file] 
(* FOR WINDOWS, REPLACE PREVIOUS LINE WITH THIS ONE:	
        val cmd = String.concat ["imdisplay ",tmp_file] *)
	val _ = TextIO.output(TextIO.stdErr,
			      "Press \"Q\" in the window to continue\n")
      in OS.Process.system cmd;()
      end
      val _ = do_it () handle _ => ()
    in OS.FileSys.remove tmp_file
    end

    val toBWGIF = toGIF toPBM
    val toRGBGIF = toGIF toPPM

    val toBWPNG = toPNG toPBM
    val toRGBPNG = toPNG toPPM
    val toPS = toPS toPSPoints
  end